==> picker-client-1.0.0-20220916.064601-19-sources.jar: com/tydic/picker/service/imp/DataSyncServiceImpl.java <==
package com.tydic.picker.service.imp;

import cn.hutool.core.codec.Base64Encoder;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.tydic.picker.dto.DataPickDTO;
import com.tydic.picker.dto.DataSyncConfigSubQueryDTO;
import com.tydic.picker.dto.PickerRecordDTO;
import com.tydic.picker.enums.ConstantEnum;
import com.tydic.picker.enums.ResultCodeEnum;
import com.tydic.picker.result.PickerResult;
import com.tydic.picker.service.DataSyncService;
import com.tydic.picker.utils.DynamicUtil;
import com.tydic.picker.utils.ElasticsearchUtil;
import com.tydic.picker.utils.Sequence;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName DataSyncServiceImpl
 * @Description 数据同步服务
 * @Author liugs
 * @Date 2022/8/15 16:48
 */
@Slf4j
public class DataSyncServiceImpl implements DataSyncService {

    /** 文档ID取值字段 */
    private static final String DOCUMENT_ID = "DOCUMENT_ID";
    /** 最大深度 */
    private static final int MAXIMUM_DEPTH = 2;

    private JdbcTemplate jdbcTemplate;

    private ElasticsearchUtil elasticsearchUtil;

    public DataSyncServiceImpl(JdbcTemplate jdbcTemplate,
                               ElasticsearchUtil elasticsearchUtil) {
        this.jdbcTemplate = jdbcTemplate;
        this.elasticsearchUtil = elasticsearchUtil;
    }

    @Override
    public PickerResult doSync(DataPickDTO dto) {
        log.info("==开始采集数据并同步ES");
        PickerResult result = new PickerResult(ResultCodeEnum.SUCCESS);
        // 记录同步信息
        Long recordId = null;
        try {
            recordId = createSyncRecord(dto);
        } catch (Exception e) {
            log.error("数据同步记录入库异常：{}", e);
        }

        String errorMessage = null;
        try {
            // 执行动态SQL并根据映射配置填充ES对象
            JSONObject esData = getMappedData(dto);
            log.debug("根据映射关系的到的数据对象：{}", JSON.toJSONString(esData, SerializerFeature.WriteMapNullValue));
            if (ObjectUtil.isNotEmpty(esData.get(DOCUMENT_ID))) {
                switch (dto.getEventTypeEnum()) {
                    case CREATE:
                        createDoc(esData, dto);
                        break;
                    case UPDATE:
                        updateDoc(esData, dto);
                        break;
                    case DELETE:
                        deleteDoc(esData, dto);
                        break;
                    default:
                        break;
                }
            } else {
                errorMessage = String.format("根据配置的文档ID取值字段[%s]未获取到文档ID", dto.getDocIdField());
            }
        } catch (Exception e) {
            log.error("同步ES数据异常：");
            e.printStackTrace();
            errorMessage = "同步ES数据异常：" + e;
        }

        // 更新同步信息
        try {
            if (StringUtils.isEmpty(errorMessage)) {
                updateRecord(recordId, ConstantEnum.PickerRecordStatusEnum.SUCCESS, ConstantEnum.PickerRecordStatusEnum.SUCCESS.getDesc());
            } else {
                updateRecord(recordId, ConstantEnum.PickerRecordStatusEnum.FAIL, errorMessage);
            }
        } catch (Exception e) {
            log.error("数据同步记录状态更新异常：{}", e);
        }

        if (StringUtils.isNotEmpty(errorMessage)) {
            result.buildFail(errorMessage);
        }
        log.info("==采集数据并同步ES完成：{}", JSON.toJSONString(result));
        return result;
    }

    /**
     * 描述 获取数据并做映射
     * @param dto
     * @return com.alibaba.fastjson.JSONObject
     * @author liugs
     * @date 2022/8/17 11:38
     */
    public JSONObject getMappedData(DataPickDTO dto) {
        // 字段映射
        JSONObject mappingRelJson = JSON.parseObject(dto.getMappingInfo());
        // 映射结果
        JSONObject mappedDataJson = new JSONObject(true);
        // 出入参参数合集
        Map<String, Object> conditionParam = dto.getConditionParam();
        // 直接使用出入参合集的参数、删除操作的条件
        if (StringUtils.isEmpty(dto.getDynamicSql())) {
            for (Map.Entry<String, Object> entry : conditionParam.entrySet()) {
                mappedDataJson.put(entry.getValue().toString(), conditionParam.get(entry.getKey()));
            }
            // 根据配置的文档ID字段取值文档ID
            mappedDataJson.put(DOCUMENT_ID, conditionParam.get(dto.getDocIdField()));
            return mappedDataJson;
        }
        // 新增、更新，查询数据库
        String dynamicSql = DynamicUtil.buildDynamicSql(dto.getDynamicSql(), conditionParam);
        log.info("顶层数据执行的取值动态语句为：{}", dynamicSql);

        Map<String, Object> dynamicSqlResult =  buildSingleData(mappedDataJson, mappingRelJson, dynamicSql, conditionParam);

        // 构建子查询
        buildSubField(mappedDataJson, dynamicSqlResult, conditionParam, dto.getSubQuery(), 0);
        mappedDataJson.put(DOCUMENT_ID, mappedDataJson.get(dto.getDocIdField()));
        return mappedDataJson;
    }

    /**
     * 描述 构建子查询
     * @param parentData            父级数据对象
     * @param parentQryResult       父级查询结果
     * @param conditionParam        出入参MAP
     * @param subQuery              子查询列表
     * @param depth                 子查询纵向深度
     * @author liugs
     * @date 2022/8/31 15:41
     */
    private void buildSubField(JSONObject parentData, Map<String, Object> parentQryResult, Map<String, Object> conditionParam,
                               List<DataSyncConfigSubQueryDTO> subQuery, int depth) {
        // 超过最大深度，子查询为空，直接返回
        if (MAXIMUM_DEPTH < depth || CollectionUtils.isEmpty(subQuery)) {
            return;
        }
        Map<String, Object> dynamicConditionMap = new HashMap<>(16);
        dynamicConditionMap.putAll(conditionParam);
        dynamicConditionMap.putAll(parentQryResult);
        // 字段映射
        JSONObject fieldMapping;
        // 构建动态查询语句
        String dynamicSql;
        // 遍历子查询列表
        for (DataSyncConfigSubQueryDTO subQueryItem : subQuery) {
            fieldMapping = JSON.parseObject(subQueryItem.getMappingInfo());
            dynamicSql = DynamicUtil.buildDynamicSql(subQueryItem.getDataFilter(), dynamicConditionMap);
            log.info("[{}]级子查询取值动态语句为：{}", depth, dynamicSql);

            ConstantEnum.DataType typeEunm = ConstantEnum.DataType.getType(subQueryItem.getDataType());

            switch (typeEunm) {
                case MULTIPLE:
                    parentData.put(subQueryItem.getParentField(), buildMultipleData(fieldMapping, dynamicSql, conditionParam));
                    break;
                case FIELD:
                    parentData.put(subQueryItem.getParentField(), buildFieldData(fieldMapping, dynamicSql, conditionParam));
                    break;
                case  EXTEND_FIELD:
                    buildExtFieldData(parentData, fieldMapping, dynamicSql);
                    break;
                case SINGLE:
                    // 本层级查询是单一对象
                    JSONObject subData = new JSONObject();
                    // 构建单一数据，并将SQL结果返回，作为子查询的条件参数，如果查询异常，将其值字段值置为null;
                    Map<String, Object> dynamicSqlResult;
                    try {
                        dynamicSqlResult = buildSingleData(subData, fieldMapping, dynamicSql, conditionParam);
                    } catch (DataAccessException e) {
                        log.error("[{}]级子执行单一对象型取值动态语句异常：{}", depth, e.getMessage());
                        parentData.put(subQueryItem.getParentField(), subData);
                        continue;
                    }
                    if (CollectionUtils.isNotEmpty(subQueryItem.getSubQuery())) {
                        buildSubField(subData, dynamicSqlResult, conditionParam, subQueryItem.getSubQuery(), ++ depth);
                        -- depth;
                    }
                    parentData.put(subQueryItem.getParentField(), subData);
                    break;
                default:
                   break;
            }
        }
    }

    /**
     * 描述 构建扩展字段数据
     * @param parentData
     * @param fieldMapping
     * @param dynamicSql
     * @return void
     * @author liugs
     * @date 2022/9/16 10:45
     */
    private void buildExtFieldData(JSONObject parentData, JSONObject fieldMapping, String dynamicSql) {
        // 执行查询语句
        List<Map<String, Object>> queryForList ;
        try {
            queryForList = jdbcTemplate.queryForList(dynamicSql);
        } catch (DataAccessException e) {
            log.error("执行动态语句获取扩展字段类型数据异常：{}", e.getMessage());
            return;
        }
        log.debug("执行动态语句获取到的数据：{}", JSON.toJSONString(queryForList));

        // 遍历查询结果，根据配置完成字段映射
        queryForList.forEach(item -> {
            for (Map.Entry<String, Object> entry : fieldMapping.entrySet()) {
                Object esKey = item.get(entry.getKey());
                if (ObjectUtil.isNotEmpty(esKey)) {
                    parentData.put(StrUtil.toCamelCase(esKey.toString()), item.get(entry.getValue()));
                }
            }
        });
    }

    /**
     * 描述  构建字段型数据
     * @param fieldMapping
     * @param dynamicSql
     * @return java.lang.Object
     * @author liugs
     * @date 2022/9/1 16:14
     */
    private Object buildFieldData(JSONObject fieldMapping, String dynamicSql, Map<String, Object> conditionParam) {
        Map<String, Object> resultDataMap;
        try {
            resultDataMap = jdbcTemplate.queryForMap(dynamicSql);
        } catch (DataAccessException e) {
           log.error("执行动态语句获取字段类型数据异常：{}", e.getMessage());
           return null;
        }
        log.debug("执行动态语句获取到的数据：{}", JSON.toJSONString(resultDataMap));

        resultDataMap.putAll(conditionParam);
        // 根据配置完成字段映射
        for (Map.Entry<String, Object> entry : fieldMapping.entrySet()) {
            Object value = resultDataMap.get(entry.getKey());
            if (ObjectUtil.isNotEmpty(value)) {
                return value;
            }
        }
        return null;
    }

    /**
     * 描述 构建多数据
     * @param fieldMapping
     * @param dynamicSql
     * @return void
     * @author liugs
     * @date 2022/8/31 14:52
     */
    private JSONArray buildMultipleData(JSONObject fieldMapping, String dynamicSql, Map<String, Object> conditionParam) {
        JSONArray subArrayData = new JSONArray();
        // 查询数据库
        List<Map<String, Object>> queryForList ;
        try {
            queryForList = jdbcTemplate.queryForList(dynamicSql);
        } catch (DataAccessException e) {
            log.error("执行动态语句获取列表类型数据异常：{}", e.getMessage());
            return subArrayData;
        }

        if (CollectionUtils.isEmpty(queryForList)) {
            log.error("语句[{}]未获取到数据列表", dynamicSql);
            return subArrayData;
        }
        // 遍历查询结果
        queryForList.forEach(item -> {
            item.putAll(conditionParam);
            // 根据映射关系构建数据对象
            JSONObject subDateItem = new JSONObject();
            for (Map.Entry<String, Object> entry : fieldMapping.entrySet()) {
                subDateItem.put(entry.getValue().toString(), item.get(entry.getKey()));
            }
            subArrayData.add(subDateItem);
        });
        return subArrayData;
    }

    /**
     * 构建单一数据
     * @param targetData
     * @param fieldMapping
     * @param dynamicSql
     */
    private Map<String, Object> buildSingleData(JSONObject targetData, JSONObject fieldMapping, String dynamicSql, Map<String, Object> conditionParam) {
        // 查询单一数据
        Map<String, Object> resultDataMap = jdbcTemplate.queryForMap(dynamicSql);
        log.debug("执行动态语句获取到的数据：{}", JSON.toJSONString(resultDataMap));
        Map<String, Object> tempDataMap = new HashMap<>(16);
        tempDataMap.putAll(conditionParam);
        tempDataMap.putAll(resultDataMap);

        // 根据配置完成字段映射
        for (Map.Entry<String, Object> entry : fieldMapping.entrySet()) {
            targetData.put(entry.getValue().toString(), tempDataMap.get(entry.getKey()));
        }
        return resultDataMap;
    }


    /**
     * 创建文档
     * @param esData
     * @param dto
     */
    private void createDoc(JSONObject esData, DataPickDTO dto) {
        elasticsearchUtil.addDocument(dto.getIndexName(), esData.remove(DOCUMENT_ID).toString(), esData);
    }

    /**
     * 更新文档
     * @param esData
     * @param dto
     */
    private void updateDoc(JSONObject esData, DataPickDTO dto) {
        elasticsearchUtil.getDocById(dto.getIndexName(), esData.getString(DOCUMENT_ID));
        elasticsearchUtil.updateDoc(dto.getIndexName(), esData.remove(DOCUMENT_ID).toString(), esData);
    }

    /**
     * 删除文档
     * @param esData
     * @param dto
     */
    private void deleteDoc(JSONObject esData, DataPickDTO dto) {
        elasticsearchUtil.deleteDocById(dto.getIndexName(), esData.getString(DOCUMENT_ID));
    }


    /**
     * 处理同步记录
     * @param dto
     * @return recordId
     */
    private Long createSyncRecord(DataPickDTO dto) {
        Long id = Sequence.nextId();
        PickerRecordDTO recordPo = new PickerRecordDTO();
        BeanUtils.copyProperties(dto, recordPo);
        recordPo.setId(id);
        recordPo.setConditionParam(JSON.toJSONString(dto.getConditionParam()));
        recordPo.setEventType(dto.getEventTypeEnum().getEventType());
        recordPo.setStartTime(new DateTime().toString("yyyy-MM-dd HH:mm:ss"));
        recordPo.setStatusDesc(ConstantEnum.PickerRecordStatusEnum.INIT.getDesc());
        recordPo.setDynamicSql(StringUtils.isEmpty(dto.getDynamicSql()) ? null : Base64Encoder.encode(dto.getDynamicSql()));
        String dynamicSql = ConstantEnum.DynamicSqlEnum.INSERT.getDynamicSql(JSON.parseObject(JSON.toJSONString(recordPo), Map.class));
        log.debug("记录数据同步SQL：{}", dynamicSql);
        if (1 > jdbcTemplate.update(dynamicSql)) {
            log.error("记录数据同步记录失败");
            return null;
        }
        return id;
    }

    /**
     * 更新记录状态
     * @param recordId
     * @param statusEnum
     * @param desc
     * @return
     */
    private void updateRecord(Long recordId, ConstantEnum.PickerRecordStatusEnum statusEnum, String desc) {
        PickerRecordDTO recordPo = new PickerRecordDTO();
        recordPo.setId(recordId);
        recordPo.setStatus(statusEnum.getState());
        recordPo.setStatusDesc(desc);
        recordPo.setFinishTime(new DateTime().toString("yyyy-MM-dd HH:mm:ss"));
        String dynamicSql = ConstantEnum.DynamicSqlEnum.UPDATE.getDynamicSql(JSON.parseObject(JSON.toJSONString(recordPo), Map.class));
        log.debug("记录数据同步SQL：{}", dynamicSql);
        if ( 1 > jdbcTemplate.update(dynamicSql)) {
            log.error("更新数据同步记录失败");
        }
    }


}

==> picker-client-1.0.0-20220919.083150-22-sources.jar: com/tydic/picker/service/imp/DataSyncServiceImpl.java <==
package com.tydic.picker.service.imp;

import cn.hutool.core.codec.Base64Encoder;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.tydic.picker.constant.PickerConstants;
import com.tydic.picker.dto.DataPickDTO;
import com.tydic.picker.dto.PickerRecordDTO;
import com.tydic.picker.enums.ConstantEnum;
import com.tydic.picker.enums.ResultCodeEnum;
import com.tydic.picker.result.PickerResult;
import com.tydic.picker.service.DataSyncService;
import com.tydic.picker.utils.DynamicSqlUtil;
import com.tydic.picker.utils.ElasticsearchUtil;
import com.tydic.picker.utils.Sequence;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.jdbc.core.JdbcTemplate;

import javax.annotation.Resource;
import java.util.Map;

/**
 * @ClassName DataSyncServiceImpl
 * @Description 数据同步服务
 * @Author liugs
 * @Date 2022/8/15 16:48
 */
@Slf4j
public class DataSyncServiceImpl implements DataSyncService {

    @Resource
    private DynamicSqlUtil dynamicSqlUtil;

    private JdbcTemplate jdbcTemplate;

    private ElasticsearchUtil elasticsearchUtil;

    public DataSyncServiceImpl(JdbcTemplate jdbcTemplate, ElasticsearchUtil elasticsearchUtil) {
        this.jdbcTemplate = jdbcTemplate;
        this.elasticsearchUtil = elasticsearchUtil;
    }

    @Override
    public PickerResult doSync(DataPickDTO dto) {
        log.info("==开始采集数据并同步ES");
        PickerResult result = new PickerResult(ResultCodeEnum.SUCCESS);
        // 记录同步信息
        Long recordId = null;
        try {
            recordId = createSyncRecord(dto);
        } catch (Exception e) {
            log.error("数据同步记录入库异常：{}", e);
        }

        String errorMessage = null;
        try {
            // 执行动态SQL并根据映射配置填充ES对象
            JSONObject esData = dynamicSqlUtil.getMappedData(dto);
            log.debug("根据映射关系的到的数据对象：{}", JSON.toJSONString(esData, SerializerFeature.WriteMapNullValue));

            if (ObjectUtil.isNotEmpty(esData.get(PickerConstants.DOCUMENT_ID))) {
                switch (dto.getEventTypeEnum()) {
                    case CREATE:
                        createDoc(esData, dto);
                        break;
                    case UPDATE:
                        updateDoc(esData, dto);
                        break;
                    case DELETE:
                        deleteDoc(esData, dto);
                        break;
                    default:
                        break;
                }
            } else {
                errorMessage = String.format("根据配置的文档ID取值字段[%s]未获取到文档ID", dto.getDocIdField());
            }
        } catch (Exception e) {
            log.error("同步ES数据异常：");
            e.printStackTrace();
            errorMessage = "同步ES数据异常：" + e;
        }

        // 更新同步信息
        try {
            if (StringUtils.isEmpty(errorMessage)) {
                updateRecord(recordId, ConstantEnum.PickerRecordStatusEnum.SUCCESS, ConstantEnum.PickerRecordStatusEnum.SUCCESS.getDesc());
            } else {
                updateRecord(recordId, ConstantEnum.PickerRecordStatusEnum.FAIL, errorMessage);
            }
        } catch (Exception e) {
            log.error("数据同步记录状态更新异常：{}", e);
        }

        if (StringUtils.isNotEmpty(errorMessage)) {
            result.buildFail(errorMessage);
        }
        log.info("==采集数据并同步ES完成：{}", JSON.toJSONString(result));
        return result;
    }



    /**
     * 创建文档
     * @param esData
     * @param dto
     */
    private void createDoc(JSONObject esData, DataPickDTO dto) {
        elasticsearchUtil.addDocument(dto.getIndexName(), esData.remove(PickerConstants.DOCUMENT_ID).toString(), esData);
    }

    /**
     * 更新文档
     * @param esData
     * @param dto
     */
    private void updateDoc(JSONObject esData, DataPickDTO dto) {
        elasticsearchUtil.getDocById(dto.getIndexName(), esData.getString(PickerConstants.DOCUMENT_ID));
        elasticsearchUtil.updateDoc(dto.getIndexName(), esData.remove(PickerConstants.DOCUMENT_ID).toString(), esData);
    }

    /**
     * 删除文档
     * @param esData
     * @param dto
     */
    private void deleteDoc(JSONObject esData, DataPickDTO dto) {
        elasticsearchUtil.deleteDocById(dto.getIndexName(), esData.getString(PickerConstants.DOCUMENT_ID));
    }


    /**
     * 处理同步记录
     * @param dto
     * @return recordId
     */
    private Long createSyncRecord(DataPickDTO dto) {
        Long id = Sequence.nextId();
        PickerRecordDTO recordPo = new PickerRecordDTO();
        BeanUtils.copyProperties(dto, recordPo);
        recordPo.setId(id);
        recordPo.setConditionParam(JSON.toJSONString(dto.getConditionParam()));
        recordPo.setEventType(dto.getEventTypeEnum().getEventType());
        recordPo.setStartTime(new DateTime().toString("yyyy-MM-dd HH:mm:ss"));
        recordPo.setStatusDesc(ConstantEnum.PickerRecordStatusEnum.INIT.getDesc());
        recordPo.setDynamicSql(StringUtils.isEmpty(dto.getDynamicSql()) ? null : Base64Encoder.encode(dto.getDynamicSql()));
        String dynamicSql = ConstantEnum.DynamicSqlEnum.INSERT.getDynamicSql(JSON.parseObject(JSON.toJSONString(recordPo), Map.class));
        log.debug("记录数据同步SQL：{}", dynamicSql);
        if (1 > jdbcTemplate.update(dynamicSql)) {
            log.error("记录数据同步记录失败");
            return null;
        }
        return id;
    }

    /**
     * 更新记录状态
     * @param recordId
     * @param statusEnum
     * @param desc
     * @return
     */
    private void updateRecord(Long recordId, ConstantEnum.PickerRecordStatusEnum statusEnum, String desc) {
        PickerRecordDTO recordPo = new PickerRecordDTO();
        recordPo.setId(recordId);
        recordPo.setStatus(statusEnum.getState());
        recordPo.setStatusDesc(desc);
        recordPo.setFinishTime(new DateTime().toString("yyyy-MM-dd HH:mm:ss"));
        String dynamicSql = ConstantEnum.DynamicSqlEnum.UPDATE.getDynamicSql(JSON.parseObject(JSON.toJSONString(recordPo), Map.class));
        log.debug("记录数据同步SQL：{}", dynamicSql);
        if ( 1 > jdbcTemplate.update(dynamicSql)) {
            log.error("更新数据同步记录失败");
        }
    }
}
